"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const utils_1 = require("./utils");
describe('modules', function () {
    describe('loading', function () {
        describe('utils', function () {
            const successAction = { type: '[Success] Some Action' };
            const failureAction = { type: '[Failure] Some Action' };
            const requestAction = { type: '[Request] Some Action' };
            const elements = [
                { type: 'A' },
                { type: 'B' },
                { type: 'C' },
                { type: 'B' },
                { type: 'A' }
            ];
            describe('removeLast', function () {
                it('should remove the forth element', function () {
                    const newElements = utils_1.removeLast(elements, element => element.type === 'B');
                    chai_1.expect(newElements).to.deep.equal([
                        { type: 'A' },
                        { type: 'B' },
                        { type: 'C' },
                        { type: 'A' }
                    ]);
                });
                it('should remove the third element', function () {
                    const newElements = utils_1.removeLast(elements, element => element.type === 'C');
                    chai_1.expect(newElements).to.deep.equal([
                        { type: 'A' },
                        { type: 'B' },
                        { type: 'B' },
                        { type: 'A' }
                    ]);
                });
                it('should remove the last element', function () {
                    const newElements = utils_1.removeLast(elements, element => element.type === 'A');
                    chai_1.expect(newElements).to.deep.equal([
                        { type: 'A' },
                        { type: 'B' },
                        { type: 'C' },
                        { type: 'B' }
                    ]);
                });
                it('should not remove any element', function () {
                    const newElements = utils_1.removeLast(elements, element => element.type === 'D');
                    chai_1.expect(newElements).to.deep.equal([
                        { type: 'A' },
                        { type: 'B' },
                        { type: 'C' },
                        { type: 'B' },
                        { type: 'A' }
                    ]);
                });
            });
            describe('getType', function () {
                it('should return the action type of a success action', function () {
                    const actionType = utils_1.getType(successAction);
                    chai_1.expect(actionType).to.be.equal('Some Action');
                });
                it('should return the action type of a failure action', function () {
                    const actionType = utils_1.getType(failureAction);
                    chai_1.expect(actionType).to.be.equal('Some Action');
                });
                it('should return the action type of a request action', function () {
                    const actionType = utils_1.getType(requestAction);
                    chai_1.expect(actionType).to.be.equal('Some Action');
                });
            });
            describe('getStatus', function () {
                it('should return "SUCCESS" status', function () {
                    const actionType = utils_1.getStatus(successAction);
                    chai_1.expect(actionType).to.be.equal('SUCCESS');
                });
                it('should return "FAILURE" status', function () {
                    const actionType = utils_1.getStatus(failureAction);
                    chai_1.expect(actionType).to.be.equal('FAILURE');
                });
                it('should return "REQUEST" status', function () {
                    const actionType = utils_1.getStatus(requestAction);
                    chai_1.expect(actionType).to.be.equal('REQUEST');
                });
            });
        });
    });
});
//# sourceMappingURL=utils.spec.js.map