"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProfileSaga = void 0;
const effects_1 = require("redux-saga/effects");
const actions_1 = require("../wallet/actions");
const actions_2 = require("./actions");
const peer_1 = require("../../lib/peer");
function createProfileSaga({ peerUrl }) {
    const peerApi = new peer_1.PeerAPI(peerUrl);
    function* profileSaga() {
        yield effects_1.takeEvery(actions_2.LOAD_PROFILE_REQUEST, handleLoadProfileRequest);
        yield effects_1.takeLatest(actions_1.CONNECT_WALLET_SUCCESS, handleWallet);
        yield effects_1.takeLatest(actions_1.CHANGE_ACCOUNT, handleWallet);
    }
    function* handleLoadProfileRequest(action) {
        const { address } = action.payload;
        try {
            const profile = yield effects_1.call(() => peerApi.fetchProfile(address));
            yield effects_1.put(actions_2.loadProfileSuccess(address, profile));
        }
        catch (error) {
            yield effects_1.put(actions_2.loadProfileFailure(address, error.message));
        }
    }
    function* handleWallet(action) {
        yield effects_1.put(actions_2.loadProfileRequest(action.payload.wallet.address));
    }
    return profileSaga;
}
exports.createProfileSaga = createProfileSaga;
//# sourceMappingURL=sagas.js.map