"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildWallet = exports.fetchManaBalance = exports.getManaBalanceQuery = void 0;
const eth_1 = require("web3x-es/eth");
const utils_1 = require("web3x-es/utils");
const eth_2 = require("../../lib/eth");
const graph_1 = require("../../lib/graph");
const chainConfiguration_1 = require("../../lib/chainConfiguration");
const getManaBalanceQuery = (address) => `query {
  accounts(where:{ id: "${address.toLowerCase()}" }) {
    id
    mana
  }
}`;
exports.getManaBalanceQuery = getManaBalanceQuery;
function fetchManaBalance(graphUrl, address) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { accounts } = yield graph_1.graphql(graphUrl, exports.getManaBalanceQuery(address));
            if (accounts.length === 0) {
                throw new Error(`No results for Graph URL "${graphUrl}" and address "${address}"`);
            }
            return parseFloat(utils_1.fromWei(accounts[0].mana, 'ether'));
        }
        catch (error) {
            return 0;
        }
    });
}
exports.fetchManaBalance = fetchManaBalance;
function buildWallet() {
    return __awaiter(this, void 0, void 0, function* () {
        const provider = yield eth_2.getConnectedProvider();
        if (!provider) {
            // This could happen if metamask is not installed
            throw new Error('Could not connect to Ethereum');
        }
        const eth = new eth_1.Eth(provider);
        const accounts = yield eth.getAccounts();
        if (accounts.length === 0) {
            // This could happen if metamask was not enabled
            throw new Error('Could not get address');
        }
        const address = accounts[0];
        const chainId = (yield eth.getId());
        const config = chainConfiguration_1.getChainConfiguration(chainId);
        const networks = {};
        for (const network of Object.keys(config.networkMapping)) {
            const networkChainId = config.networkMapping[network];
            const networkConfiguration = chainConfiguration_1.getChainConfiguration(networkChainId);
            networks[network] = {
                chainId: networkChainId,
                mana: yield fetchManaBalance(networkConfiguration.manaGraphURL, address.toString())
            };
        }
        return {
            address: address.toString(),
            providerType: eth_2.getConnectedProviderType(),
            networks: networks,
            network: config.network,
            chainId
        };
    });
}
exports.buildWallet = buildWallet;
//# sourceMappingURL=utils.js.map